#!/bin/bash
###########################################################
# convert.sh
# script to convert the selected jpeg to mpeg
# for mplayer plugin script imageviewer modified
# needs : netpbm-progs, mjpegtools, [modified] test_stillimage from dvb-driver, [irexec setup]
# History:
# 11/2003    okx: added cache browser stuff, reworked config, need identify from imagemagick againe, we need the pixes sizes, we can rotate now
# 16.05.2003 okx :removed "identify" replaced by file -i -L -b
#            removed image pixel detection, since using pnmcut we always generate 704x576
# 02-04.2003 okx(at)gmx(dot)de reworked to use only mpegtools and pnm tools, no imagemagic/mpeg2encode needed only old identify used from X11 sw.
#
# 31.01.03 0.0.7 KTB // basename 
#			thanks to Andreas Kool
# 31.01.03 0.0.6 KTB // Rotation
# 28.01.03 0.0.5 KTB // 
# 27.01.03 0.0.4 KTB // DEBUG
# 27.01.03 0.0.3 KTB // caching 
# 23.01.03 0.0.2 KTB // geometry 
# 22.01.03 0.0.1 KTB // Initial Version
###########################################################
#set -vx

#CONVERT_TEMPDIR=/tmp/images
#CONVERT_LOGFILE=/tmp/convert.log
#CONVERT_MPEGFILE=/tmp/image.mpg

#CONVERT_TEMPDIR="${CONVERT_TEMPDIR:-"/tmp/images"}"
#CONVERT_LOGFILE="${CONVERT_LOGFILE:-"/tmp/convert.log"}"
#CONVERT_MPEGFILE="${CONVERT_MPEGFILE:-"/tmp/image.mpg"}"

if test "$2" == ""; then CONVERT_TEMPDIR="/tmp/image"; else CONVERT_TEMPDIR=$2; fi
CONVERT_LOGFILE=$CONVERT_TEMPDIR/convert.log
CONVERT_MPEGFILE=$CONVERT_TEMPDIR/image.mpg

IDENTIFY="${IDENTIFY:-"identify"}"  #uuaaarg this is from image magic, sic! needed to get x and y pixel sizes
BACKGROUND="$0.background"
[ ! -e $BACKGROUND ] && echo "Error $BACKGROUND does not exist, it shoud be a 704x576 black pnm file" && exit 1

mkdir -p $CONVERT_TEMPDIR 

function logit () {
#  logger "convert.sh: $1"
  echo "$1" | tee -a $CONVERT_LOGFILE
  return;
}

DATE=`date +%d.%m.%y-%T`
echo "$0: $DATE: startet ..." > $CONVERT_LOGFILE
DIR=$(dirname "$1")
FILE=$(basename "$1")
NEWDIR="${CONVERT_TEMPDIR}${DIR}"
NEWFILE="${NEWDIR}/$FILE.mpg"
CMDFILE="${NEWDIR}/$FILE.cnv"
PARFILE="${NEWDIR}/$FILE.par"
logit "CMDFILE: $CMDFILE"
logit "PARFILE: $PARFILE"
logit "IMAGEFILE: $1"; 
logit "CONVERT_MPEGFILE: $NEWFILE";

ROTATE="NO"
ORIGINAL="NO"
NOCOPY=0
if test "$3" == "Right"; then ROTATE="YES"; FLIP="-rotate270"; fi
if test "$3" == "Left"; then ROTATE="YES"; FLIP="-rotate90"; fi
if test "$3" == "Original"; then ORIGINAL="YES"; fi
if test "$3" == "nocopy"; then NOCOPY=1; fi
echo "$3" > "$CMDFILE"

FILE_DETECT=$(file -i -L -b "$1" 2>/dev/null)
FILE_TYPE=$(echo "$FILE_DETECT"|cut -f1 -d";"|cut -f1 -d"/")
FILE_SUB_TYPE=$(echo "$FILE_DETECT"|cut -f1 -d";"|cut -f2 -d"/")

#new cache browser code
case "$FILE_DETECT" in
video/mpeg)
	logit "Experimental mpeg still (cache browser) used, video/mpeg file type detected"
	MPEG_IN_SIZE=$(du -h -k "$1"|awk '{ print $1 }')
	[ $MPEG_IN_SIZE -gt 10 -a $MPEG_IN_SIZE -lt 230 ] && rm -f "$CONVERT_MPEGFILE" && cp -f "$1" "$CONVERT_MPEGFILE" | tee -a $CONVERT_LOGFILE && exit 0
	exit 1
	;;
esac

#end cache browser code


case "$FILE_TYPE" in
image)
        logit  "Image file type detected"
        ;;
*)
        logit "Not an image file : "$1""
        exit 1
        ;;
esac


if test -f "$NEWFILE" && test "$ROTATE" == "YES" || test "$ORIGINAL" == "YES"; then rm -f "$NEWFILE"; fi

if [ ! -f "$NEWFILE" ]  
#if mpg does not exist in cache
then
    [ ! -d "$NEWDIR" ] && mkdir -p -v "$NEWDIR" | tee -a $CONVERT_LOGFILE

    logit  "CONVERT: $FILE --> $NEWFILE";

    TEMP1=`echo $1 | sed -e "s/\///g"`
    TEMP2=`identify "$1" | sed -e "s/\///g"`

    GEOMETRY=`echo $TEMP2 | sed -e "s/$TEMP1//g" | cut -d" " -f3 | cut -d"+" -f1`
    X_RES_TMP=`echo $GEOMETRY | cut -d"x" -f1`
    Y_RES_TMP=`echo $GEOMETRY | cut -d"x" -f2 | cut -d"=" -f1`
    echo "$X_RES_TMP" >"$PARFILE"
    echo "$Y_RES_TMP">>"$PARFILE"

    case "$FILE_SUB_TYPE" in
	jpeg)
	TO_PNM=jpegtopnm
	;;
	tiff)
	TO_PNM=tifftopnm
	;;
	x-bmp)
	TO_PNM=bmptoppm
	;;
	x-png)
	TO_PNM=pngtopnm
	;;
	x-portable-pixmap)
	TO_PNM=cat
	;;
	gif)
	TO_PNM=giftopnm
	;;
	*)
	logit "File "$FILE_TYPE/$FILE_SUB_TYPE" is not of supported type , EXIT 1"
	exit 1
	;;
    esac
    logit "File will be converted using: $TO_PNM"

  if test "$ORIGINAL" == "YES"
  then
     logit "Orignal"
  fi
  if test "$ROTATE" == "YES"
  then
     logit "Flip Image"
     X_RES=$Y_RES_TMP
     Y_RES=$X_RES_TMP
  else
     logit "don't Flip Image"
     X_RES=$X_RES_TMP
     Y_RES=$Y_RES_TMP
   fi

    logit "X-RES: $X_RES | Y-RES: $Y_RES"

#TO make a real calc for the asp corrector
    if [ "$X_RES" -lt "$Y_RES" ]
	then
	SCALE="pnmscale -xsize=704 -ysize=576"
    else
	SCALE="pnmscale -xsize=432 -ysize=576"
	FIT_SCREEN="| pnmcomp -align=center - $BACKGROUND"
    fi

   if [ "$ROTATE" == "YES" -a "$X_RES" -gt "$Y_RES" ];then
    $TO_PNM "$1" | pnmflip $FLIP | pnmscale -xsize=704 -ysize=576 | ppmntsc --pal | \
       ppmtoy4m -F 25:1 -A 4:3 | mpeg2enc -f 7 -T 90 -F 3 -np -a 2 -o "$NEWFILE" | \
       tee -a $CONVERT_LOGFILE
   elif [ "$ROTATE" == "YES" -a "$X_RES" -lt "$Y_RES" ];then
    $TO_PNM "$1" | pnmflip $FLIP | pnmscale -xsize=432 -ysize=576 | pnmcomp -align=center - $BACKGROUND | \
       ppmntsc --pal  | ppmtoy4m  -F 25:1 -A 4:3 | mpeg2enc -f 7 -T 90 -F 3 -np -a 2 -o "$NEWFILE" | \
       tee -a $CONVERT_LOGFILE
   elif [ "$ROTATE" == "NO" -a "$X_RES" -gt "$Y_RES" ];then
    $TO_PNM "$1" | pnmscale -xsize=704 -ysize=576 | ppmntsc --pal | \
       ppmtoy4m -F 25:1 -A 4:3 | mpeg2enc -f 7 -T 90 -F 3 -np -a 2 -o "$NEWFILE" | \
       tee -a $CONVERT_LOGFILE
   elif [ "$ROTATE" == "NO" -a "$X_RES" -le "$Y_RES" ];then
    $TO_PNM "$1" | pnmscale -xsize=432 -ysize=576 | pnmcomp -align=center - $BACKGROUND | \
       ppmntsc --pal  | ppmtoy4m  -F 25:1 -A 4:3 | mpeg2enc -f 7 -T 90 -F 3 -np -a 2 -o "$NEWFILE" | \
       tee -a $CONVERT_LOGFILE
    fi

else
    logit "$NEWFILE exists ... convert not required";

    TEMP1=`echo $1 | sed -e "s/\///g"`
    TEMP2=`identify "$1" | sed -e "s/\///g"`

    GEOMETRY=`echo $TEMP2 | sed -e "s/$TEMP1//g" | cut -d" " -f3 | cut -d"+" -f1`
    X_RES_TMP=`echo $GEOMETRY | cut -d"x" -f1`
    Y_RES_TMP=`echo $GEOMETRY | cut -d"x" -f2 | cut -d"=" -f1`
    echo "$X_RES_TMP" >"$PARFILE"
    echo "$Y_RES_TMP">>"$PARFILE"
fi

#logit 


if [ ! -f "$NEWFILE" ] 
then 
  logit "$DATE: ERROR Convert Image-to-MPEG2-STILL not successfull for : $IMAGEFILE"; 
else
  logit "$DATE: successfull ...";

#kind a dirty break if we dont want to copy resulting file
# but pregenrate a file
[ $NOCOPY -eq 1 ] && exit 0

  if [ $(wc -c "$NEWFILE"|sed 's/^ *//'|cut -f1 -d" ") -gt 100 ]
       then
	echo "doing the copy to $CONVERT_MPEGFILE from $NEWFILE"
	rm -f $CONVERT_MPEGFILE
	cp -f "$NEWFILE" $CONVERT_MPEGFILE | tee -a $CONVERT_LOGFILE
       else
	echo "Error generated mpeg is smaler than 100 byte, so not used!!"
	echo "generating error-msg image.."
	uudecode $0.error-message -o $CONVERT_MPEGFILE
	uudecode $0.error-message -o $CONVERT_MPEGFILE
  fi
fi


