/*
 * Image plugin to VDR (C++)
 *
 * (C) 2003 Kai Tobias Burwieck <kai@burwieck.net>
 *
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 */

#include <getopt.h>
#include <malloc.h>
#include <stdlib.h>
#include <vdr/plugin.h>

#include "common.h"
#include "setup.h"
#include "setup-image.h"
#include "data-image.h"
#include "menu-image.h"
#include "i18n.h"

static const char *VERSION        = "0.0.6";
static const char *DESCRIPTION    = "A Image Viewer plugin";
static const char *MAINMENUENTRY  = "Image";

class cPluginImage : public cPlugin {
public:
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return DESCRIPTION; }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Start(void);
  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
  virtual cOsdMenu *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
  };

const char *cPluginImage::CommandLineHelp(void)
{
  static char *help_str=0;
  
  free(help_str);    //                                     for easier orientation, this is column 80|
  asprintf(&help_str,"  -m CMD,   --mount=CMD    use CMD to mount/unmount/eject mp3 sources\n"
                     "                           (default: %s)\n"
                     "  -C CMD,   --convert=CMD  use CMD when converting Images\n"
                     "                           (default: %s)\n",
                     mountscript,
                     ImageCmd
                     );
  return help_str;
}

bool cPluginImage::ProcessArgs(int argc, char *argv[])
{
  static struct option long_options[] = {
      { "mount",    required_argument, NULL, 'm' },
      { "convert",  required_argument, NULL, 'C' },
      { NULL }
    };

  int c, option_index = 0;
  while((c=getopt_long(argc,argv,"m:C:",long_options,&option_index))!=-1) {
    switch (c) {
      case 'm': mountscript=optarg; break;
      case 'C': ImageCmd=optarg; break;
      default:  return false;
      }
    }
  return true;
}

bool cPluginImage::Start(void)
{
  i18n_name=Name();
  ImageSources.Load(AddDirectory(ConfigDirectory(),"imagesources.conf"));
  if(ImageSources.Count()<1) {
    fprintf(stderr,"vdr: you must have defined at least one source in imagesources.conf\n");
    return false;
    }
  RegisterI18n(Phrases);
  return true;
}

cOsdMenu *cPluginImage::MainMenuAction(void)
{
  return new cMenuImageBrowse;
}

cMenuSetupPage *cPluginImage::SetupMenu(void)
{
  return new cMenuSetupImage;
}

bool cPluginImage::SetupParse(const char *Name, const char *Value)
{
  if(!strcasecmp(Name, "SlideShow")) ImageSetup.SlideShow = atoi(Value);
  if(!strcasecmp(Name, "SSsec")) ImageSetup.SSsec = atoi(Value);
  if(!strcasecmp(Name, "TempDir")) strn0cpy(ImageSetup.TempDir,Value,128);
  if(!strcasecmp(Name, "RemoteLayout")) ImageSetup.RemoteLayout = atoi(Value);
  if(!strcasecmp(Name, "ShowDate")) ImageSetup.ShowDate = atoi(Value);
  if(!strcasecmp(Name, "AutoRepeat")) ImageSetup.AutoRepeat = atoi(Value);
  if(!strcasecmp(Name, "UseDeviceStillPicture")) ImageSetup.UseDeviceStillPicture = atoi(Value);
  else return false;
  return true;
}

VDRPLUGINCREATOR(cPluginImage); // Don't touch this!
