/*

 * Image plugin to VDR (C++)

 *

 * (C) 2003 Kai Tobias Burwieck <kai@burwieck.net>

 *

 * This code is free software; you can redistribute it and/or

 * modify it under the terms of the GNU General Public License

 * as published by the Free Software Foundation; either version 2

 * of the License, or (at your option) any later version.

 *

 * This code is distributed in the hope that it will be useful,

 * but WITHOUT ANY WARRANTY; without even the implied warranty of

 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

 * GNU General Public License for more details.

 *

 * You should have received a copy of the GNU General Public License

 * along with this program; if not, write to the Free Software

 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html

 */



#ifndef ___DVB_IMAGE_H

#define ___DVB_IMAGE_H



#include <vdr/player.h>

#include <vdr/thread.h>



/* NEU */
#include <linux/dvb/video.h>

class cImagePlayer;

class cSlideShow;



class cImagePlayerControl : public cControl {

private:

  const char *FileName;

public:

  cImagePlayer *imageplayer;

  time_t startime, convertime, t1, t2; 

  bool ss, cstarted;

  const char *filename;

  int maxnum, aktnum, t3;

  cImagePlayerControl(cSlideShow *sshow);

  virtual ~cImagePlayerControl();

  bool Active(void);

  void Stop(void);

  void Pause(void);

  void Play(void);

  void NextImage(int Step);

  void PrevImage(int Step);

  void GotoImage(int Pict);

  void RFlipImage(void);

  void LFlipImage(void);

  void OriginalImage(void);

  void Jump(void);

  int  ImageCount(void);

  void IncSlideTime(void);

  void DecSlideTime(void);

  bool ConvertZoom(const char *filename, char *change, int xRes, int yRes, int xPos, int yPos);

};



#endif //__DVB_IMAGE_H 











#include <ctype.h>

#include <stdlib.h>

#include <stdio.h>

#include <stdarg.h>

#include <fcntl.h>

#include <errno.h>

#include <sys/ioctl.h>

#include <sys/types.h>

#include <unistd.h>

#include <signal.h>

#include <wait.h>

#include <math.h>

#include <locale.h>



#include <vdr/device.h>

#include <vdr/status.h>

#include "common.h"

#include "player-image.h"

#include "setup-image.h"

#include "data-image.h"



const char *ImageCmd     = "convert.sh";
const char *ImageCmdZoom = "convert_zoom.sh";


//----------cImagePlayer-------------



class cImagePlayer : public cPlayer {

private:

  bool active, sstarted, started;

  int inpipe[2], outpipe[2];

  cSlideShow *slideshow;

  cImage *current;

  bool Fork(void);

  void ClosePipe(void);

  void ImageControl(const char *format, ...);

protected:

  cOsdBase *dvbOsd;

  virtual void Activate(bool On);

public:

  cImagePlayer(cSlideShow *sshow, const char *FileName);

  virtual ~cImagePlayer();

  bool Active(void) { return active; }

  const char *filename;

  char *change;

  int max, num, pid;

  bool cstarted;

  void Play(void);

  void NextImage(int Step);

  void PrevImage(int Step);

  void GotoImage(int Pict);

  int  ImageCount(void);

  void IncSlideTime(void);

  void DecSlideTime(void);

  void ChangeImage(char *change);

  bool Convert(const char *filename, char *c);

  bool ConvertZoom(const char *filename, char *change, int xRes, int yRes, int xPos, int yPos);

  bool ShowImage(void);

  void send_pes_packet(unsigned char *data, int len, int timestamp);

  };





cImagePlayer::cImagePlayer(cSlideShow *sshow, const char *FileName)

:cPlayer()

{

	num=0;

	inpipe[0]=inpipe[1]=-1;outpipe[0]=outpipe[1]=-1;pid=-1;

	slideshow=sshow;

	if(slideshow) {

		filename=slideshow->First()->Name();

		max=slideshow->Count();

		current=slideshow->First();

		num++;

	}

	active=true;

	started=false;

	sstarted=false;

}



cImagePlayer::~cImagePlayer()

{

  kill(pid,SIGTERM);

  waitpid(pid,0,0);

  ClosePipe();

  Detach();

}



void cImagePlayer::Activate(bool On)

{

  if(On) {

   if(slideshow && !started) {

    Convert(filename, "");

   }

  }

}



void cImagePlayer::ClosePipe(void)

{

  if(inpipe[0]>=0) { close(inpipe[0]); inpipe[0]=-1;}

  if(inpipe[1]>=0) { close(inpipe[1]); inpipe[1]=-1;}

  if(outpipe[0]>=0) { close(outpipe[0]); outpipe[0]=-1;}

  if(outpipe[1]>=0) { close(outpipe[1]); outpipe[1]=-1;}

}



void cImagePlayer::Play(void)

{

    if(current) {

	if(!cstarted) {

	  ShowImage();

        }

     }

}



void cImagePlayer::NextImage(int Step)

{
    if(current && num < max) {

	cImage *newcurr;

	for ( int i = 0 ; (i < Step) && (num < max) ; i++ )
	{
		newcurr=slideshow->cList<cImage>::Next(current);
		current=newcurr;
		num++;
	}

    filename=current->Name();

	Convert(filename, "");

    }
	else if ( ImageSetup.AutoRepeat ) {
		cImage *newcurr;

		newcurr=slideshow->cList<cImage>::First();
		current=newcurr;
		num = 1;

		filename=current->Name();

		Convert(filename, "");
	}
}



void cImagePlayer::PrevImage(int Step)

{
    if(current && num > 1) {

		cImage *newcurr;

		for ( int i = 0 ; (i < Step) && (num > 1); i++ )
		{
			newcurr=slideshow->cList<cImage>::Prev(current);
			
			current=newcurr;

			num--;
		}

		filename=current->Name();

		Convert(filename, "");

     }
	else if ( ImageSetup.AutoRepeat ) {
		cImage *newcurr;

		newcurr=slideshow->cList<cImage>::Last();
		current=newcurr;
		num = max;

		filename=current->Name();

		Convert(filename, "");
	}

}


void cImagePlayer::GotoImage(int Pict)
{

    if(current && Pict > 0 && Pict < max) {

	cImage *newcurr;

	newcurr=slideshow->cList<cImage>::First();
	current=newcurr;
	num = 0;

	for ( int i = 0 ; i < Pict ; i++ )
	{
		newcurr=slideshow->cList<cImage>::Next(current);
		
		current=newcurr;

		num++;
	}

	filename=current->Name();

	Convert(filename, "");

     }

}


int  cImagePlayer::ImageCount(void)

{
	return max;
}



void cImagePlayer::IncSlideTime(void)
{
	if ( ImageSetup.SSsec < 120 )
	{
		ImageSetup.SSsec++;
	}
}

void cImagePlayer::DecSlideTime(void)
{
	if ( ImageSetup.SSsec > 2 )
	{
		ImageSetup.SSsec--;
	}
}

void cImagePlayer::ChangeImage(char *change)

{

    if(change) {

	if(current) {

	   filename=current->Name();

	   if(change) Convert(filename, change);

	}

    }

}



bool cImagePlayer::Convert(const char *filename, char *change)

{

	cstarted=true;

	if(inpipe[0]>=0) { close(inpipe[0]); inpipe[0]=-1;}

	if(inpipe[1]>=0) { close(inpipe[1]); inpipe[1]=-1;}

	if(outpipe[0]>=0) { close(outpipe[0]); outpipe[0]=-1;}

	if(outpipe[1]>=0) { close(outpipe[1]); outpipe[1]=-1;}

	pid=fork();

	if(pid==-1) {

		esyslog("ERROR: convert failed: (%d) %s",errno,strerror(errno));

		return false;

	}

	if(pid==0) {

	 dsyslog("image: script startet (pid=%d)",getpid());

	 char cmd[64+PATH_MAX*2];

         snprintf(cmd,sizeof(cmd),"%s \"%s\" \"%s\" %s",ImageCmd,filename,ImageSetup.TempDir,change?change:"");

	 execle("/bin/sh","sh","-c",cmd,0,environ);

	 esyslog("ERROR: exec failed for %s: (%d) %s",cmd,errno,strerror(errno));

	 exit(127);

	}

return true;

}

bool cImagePlayer::ConvertZoom(const char *filename, char *change, int xRes, int yRes, int xPos, int yPos)

{

	cstarted=true;

	if(inpipe[0]>=0) { close(inpipe[0]); inpipe[0]=-1;}

	if(inpipe[1]>=0) { close(inpipe[1]); inpipe[1]=-1;}

	if(outpipe[0]>=0) { close(outpipe[0]); outpipe[0]=-1;}

	if(outpipe[1]>=0) { close(outpipe[1]); outpipe[1]=-1;}

	pid=fork();

	if(pid==-1) {

		esyslog("ERROR: convert failed: (%d) %s",errno,strerror(errno));

		return false;

	}

	if(pid==0) {

	 dsyslog("image: script startet (pid=%d)",getpid());

	 char cmd[128+PATH_MAX*2];

     snprintf(cmd,sizeof(cmd),"%s \"%s\" \"%s\" %s %d %d %d %d",
		 ImageCmdZoom,filename,ImageSetup.TempDir,
		 change?change:"original",xRes,yRes,xPos,yPos);

	 dsyslog("%s",cmd);

	 execle("/bin/sh","sh","-c",cmd,0,environ);

	 esyslog("ERROR: exec failed for %s: (%d) %s",cmd,errno,strerror(errno));

	 exit(127);

	}

return true;

}



bool cImagePlayer::ShowImage(void)

{

  if(!sstarted) 

   {
    uchar *buffer;

    int	filefd;

    struct stat	st;
    char tempfile[138];

    struct video_still_picture sp;

    snprintf(tempfile,sizeof(tempfile),"%s%s",ImageSetup.TempDir,"/image.mpg");

    if ( (filefd = open(tempfile,O_RDONLY)) >= 0 )
    {
		dsyslog("file %s opened",tempfile);
			
		fstat(filefd,&st);
			
		sp.iFrame = (char *) malloc(st.st_size);
		sp.size   = st.st_size;
		
		if ( sp.iFrame )
		{
			dsyslog("%d bytes fread from %s",
				read(filefd,sp.iFrame,sp.size),
				tempfile);
			
			buffer=(uchar *)sp.iFrame;

			if ( ImageSetup.UseDeviceStillPicture )
			{
				DeviceStillPicture(buffer,sp.size);
			}
			else
			{
				for (int i=1;i<=25;i++) 
					send_pes_packet (buffer,sp.size,i);
			}

			free(sp.iFrame);
		} else {

			fprintf(stderr,"Fehler beim Allokieren von %d bytes \n",sp.size);

		}

		close(filefd);
    } else {

		fprintf(stderr,"Fehler beim Lesen der Datei /tmp/image.mpg \n");

    }

    sstarted=false;

   }

  else

   {

    sstarted=true;

   }

return true;

}



void cImagePlayer::send_pes_packet (unsigned char *data, int len, int timestamp)

{

#define PES_MAX_SIZE 2048

	int ptslen = timestamp ? 5 : 1;

	static unsigned char pes_header[PES_MAX_SIZE];

	

	pes_header[0] = pes_header[1] = 0;

	pes_header[2] = 1;

	pes_header[3] = 0xe0;



	while (len > 0)

	{

		int payload_size = len;

		if (6 + ptslen + payload_size > PES_MAX_SIZE)

			payload_size =PES_MAX_SIZE - (6 + ptslen);

	

		pes_header[4] = (ptslen + payload_size) >> 8;

		pes_header[5] = (ptslen + payload_size) & 255;



		if (ptslen == 5)

		{

			int x;

			x = (0x02 << 4) | (((timestamp >> 30) & 0x07) << 1) | 1;

			pes_header[8] = x;

			x = ((((timestamp >> 15) &0x7fff) << 1) | 1);

			pes_header[7] = x >> 8;

			pes_header[8] = x & 255;

			x = ((((timestamp) & 0x7fff) < 1) | 1);

			pes_header[9] = x >> 8;

			pes_header[10] = x & 255;

		}

		else

		{

			pes_header[6] = 0x0f;

		}



		memcpy (&pes_header[6 + ptslen], data, payload_size);

		PlayVideo(pes_header, 6 + ptslen + payload_size);



		len -= payload_size;

		data += payload_size;

		ptslen = 1;

	}

}





//----------cImagePlayerControl---------------



cImagePlayerControl::cImagePlayerControl(cSlideShow *sshow)

:cControl(imageplayer = new cImagePlayer(sshow, FileName=sshow->First()->Name()))

{

	startime=time(NULL);

}



cImagePlayerControl::~cImagePlayerControl()

{

  Stop();

}



bool cImagePlayerControl::Active(void)

{

  if(ss) {

   t2=time(NULL);

   t3=t2-startime;

   if(t3 >= ImageSetup.SSsec) {

    if(/*imageplayer->num < imageplayer->max &&*/ !imageplayer->cstarted) { 

      startime=time(NULL);

      NextImage(1);

    }

   }

  }

  if(imageplayer->cstarted) {

    startime=time(NULL);

    waitpid(imageplayer->pid,0,WNOHANG);

    if(kill(imageplayer->pid,0)<0 && errno==ESRCH) {

       imageplayer->cstarted=false;

       imageplayer->Play();

    }

  }

  cstarted=imageplayer->cstarted; filename=imageplayer->filename; 

  aktnum=imageplayer->num; maxnum=imageplayer->max;

  return imageplayer && imageplayer->Active();

}



void cImagePlayerControl::Stop(void)

{

  delete imageplayer; imageplayer=0;

}



void cImagePlayerControl::Pause(void)

{

 // for Slideshow

}



void cImagePlayerControl::Play(void)

{

 // for SlideShow

}  



void cImagePlayerControl::NextImage(int Step)

{

  if (imageplayer) imageplayer->NextImage(Step);

}



void cImagePlayerControl::PrevImage(int Step)

{

  if (imageplayer) imageplayer->PrevImage(Step);

}


void cImagePlayerControl::GotoImage(int Pict)

{

  if (imageplayer) imageplayer->GotoImage(Pict);

}


int  cImagePlayerControl::ImageCount(void)

{

  if (imageplayer) return imageplayer->ImageCount();

  return 0;

}


void cImagePlayerControl::RFlipImage(void)

{

  if (imageplayer) imageplayer->ChangeImage("Right");

}



void cImagePlayerControl::LFlipImage(void)

{

  if (imageplayer) imageplayer->ChangeImage("Left");

}



void cImagePlayerControl::OriginalImage(void)

{

  if (imageplayer) imageplayer->ChangeImage("Original");

}



void cImagePlayerControl::Jump(void)

{
}


void cImagePlayerControl::IncSlideTime(void)
{

  if (imageplayer) imageplayer->IncSlideTime();

}
	

void cImagePlayerControl::DecSlideTime(void)
{

  if (imageplayer) imageplayer->DecSlideTime();

}
	
bool cImagePlayerControl::ConvertZoom(const char *filename, char *change, int xRes, int yRes, int xPos, int yPos)
{

  if (imageplayer) return imageplayer->ConvertZoom(filename,change,xRes,yRes,xPos,yPos);

  return 0;

}
